@@ -9,12 +9,12 @@ linearity <- function(input, representation = c("H", "V")) {
     validcdd(input, representation)
 
     if (is.character(input)) {
-        out <- .Call("impliedLinearity", input, h, PACKAGE = "rcdd")
+        out <- .Call(C_impliedLinearity, input, h)
     } else {
         if (! is.numeric(input))
             stop("input must be numeric or character")
         storage.mode(input) <- "double"
-        out <- .Call("impliedLinearity_f", input, h, PACKAGE = "rcdd")
+        out <- .Call(C_impliedLinearity_f, input, h)
     }
 
     return(out)
